@@ -8,8 +8,8 @@ use vars qw($TYPEMAP_KIND $class_part $class_decl $star);
 # Dev versions will have a _0xx suffix.
 # We eval the $VERSION to accommodate dev version numbering as described in
 # perldoc perlmodstyle
-our $VERSION = '0.38';
-$VERSION = eval $VERSION;  ## no critic (eval)
+our $VERSION = '0.39';
+# $VERSION = eval $VERSION;  ## no critic (eval)
 
 # Parse::RecDescent 1.90 and later have an incompatible change
 # 'The key of an %item entry for a repeated subrule now includes
@@ -509,3 +509,10 @@ sub strip_ellipsis {
 }
 
 1;
+
+__END__
+=head1 Inline::CPP::grammar
+
+All functions are internal.  No documentation necessary.
+
+=cut
