@@ -5,13 +5,13 @@ package Inline::CPP::Config;
 use strict;
 use warnings;
 
-our $VERSION = '0.54_001';
+our $VERSION = '0.38_004';
 $VERSION = eval $VERSION; ## no critic (eval)
 
 
 # DO NOT MANUALLY ALTER THE FOLLOWING TWO LINES: Makefile.PL locates them by
 # matching their syntax and identifier names.
-our $compiler = 'g++ -xc++  -D_FILE_OFFSET_BITS=64';
+our $compiler = 'g++';
 our $libs     = '-lstdc++';
 
 
